@@ -14,16 +14,15 @@
 #    under the License.
 
 import mock
-from oslotest import base
-
 from six.moves import range  # noqa
 
+from os_win import constants
 from os_win import exceptions
+from os_win.tests import test_base
 from os_win.utils.compute import vmutils
-from os_win.utils import constants
 
 
-class VMUtilsTestCase(base.BaseTestCase):
+class VMUtilsTestCase(test_base.OsWinBaseTestCase):
     """Unit tests for the Hyper-V VMUtils class."""
 
     _FAKE_VM_NAME = 'fake_vm'
@@ -66,31 +65,25 @@ class VMUtilsTestCase(base.BaseTestCase):
     _VIRTUAL_SYSTEM_TYPE_REALIZED = 'Microsoft:Hyper-V:System:Realized'
 
     def setUp(self):
+        super(VMUtilsTestCase, self).setUp()
         self._vmutils = vmutils.VMUtils()
         self._vmutils._conn = mock.MagicMock()
         self._vmutils._jobutils = mock.MagicMock()
         self._vmutils._pathutils = mock.MagicMock()
 
-        super(VMUtilsTestCase, self).setUp()
+    def test_vs_man_svc(self):
+        expected = self._vmutils._conn.Msvm_VirtualSystemManagementService()[0]
+        self.assertEqual(expected, self._vmutils._vs_man_svc)
 
-    @mock.patch('os_win.utils.hostutils.HostUtils'
-                '.check_min_windows_version')
-    @mock.patch.object(vmutils, 'sys')
-    @mock.patch.object(vmutils, 'wmi', create=True)
-    def test_serial_port_setting_data_win_version_10(self, mock_wmi, mock_sys,
-                                                     mock_check_version):
-        mock_sys.platform = 'win32'
-        mock_check_version.return_value = True
-        _vmutils = vmutils.VMUtils()
-
-        self.assertEqual("Msvm_SerialPortSettingData",
-                         _vmutils._SERIAL_PORT_SETTING_DATA_CLASS)
+    def test_vs_man_svc_cached(self):
+        self._vmutils._vs_man_svc_attr = mock.sentinel.fake_svc
+        self.assertEqual(mock.sentinel.fake_svc, self._vmutils._vs_man_svc)
 
     def test_get_vm_summary_info(self):
         self._lookup_vm()
-        mock_svc = self._vmutils._conn.Msvm_VirtualSystemManagementService()[0]
 
         mock_summary = mock.MagicMock()
+        mock_svc = self._vmutils._vs_man_svc
         mock_svc.GetSummaryInformation.return_value = (self._FAKE_RET_VAL,
                                                        [mock_summary])
 
@@ -132,7 +125,8 @@ class VMUtilsTestCase(base.BaseTestCase):
     def test_set_vm_memory_dynamic(self):
         self._test_set_vm_memory_dynamic(2.0)
 
-    def _test_set_vm_memory_dynamic(self, dynamic_memory_ratio):
+    def _test_set_vm_memory_dynamic(self, dynamic_memory_ratio,
+                                    mem_per_numa_node=None):
         mock_s = self._vmutils._conn.Msvm_VirtualSystemSettingData()[0]
         mock_s.SystemType = 3
 
@@ -141,16 +135,42 @@ class VMUtilsTestCase(base.BaseTestCase):
 
         self._vmutils._set_vm_memory(mock_vmsetting,
                                      self._FAKE_MEMORY_MB,
+                                     mem_per_numa_node,
                                      dynamic_memory_ratio)
 
         self._vmutils._jobutils.modify_virt_resource.assert_called_once_with(
             mock_s)
 
+        if mem_per_numa_node:
+            self.assertEqual(mem_per_numa_node,
+                             mock_s.MaxMemoryBlocksPerNumaNode)
         if dynamic_memory_ratio > 1:
             self.assertTrue(mock_s.DynamicMemoryEnabled)
         else:
             self.assertFalse(mock_s.DynamicMemoryEnabled)
 
+    def test_set_vm_vcpus(self):
+        self._check_set_vm_vcpus()
+
+    def test_set_vm_vcpus_per_vnuma_node(self):
+        self._check_set_vm_vcpus(vcpus_per_numa_node=1)
+
+    def _check_set_vm_vcpus(self, vcpus_per_numa_node=None):
+        procsetting = mock.MagicMock()
+        mock_vmsetting = mock.MagicMock()
+        mock_vmsetting.associators.return_value = [procsetting]
+
+        self._vmutils._set_vm_vcpus(mock_vmsetting,
+                                    self._FAKE_VCPUS_NUM,
+                                    vcpus_per_numa_node,
+                                    limit_cpu_features=False)
+
+        self._vmutils._jobutils.modify_virt_resource.assert_called_once_with(
+            procsetting)
+        if vcpus_per_numa_node:
+            self.assertEqual(vcpus_per_numa_node,
+                             procsetting.MaxProcessorsPerNumaNode)
+
     def test_soft_shutdown_vm(self):
         mock_vm = self._lookup_vm()
         mock_shutdown = mock.MagicMock()
@@ -183,6 +203,17 @@ class VMUtilsTestCase(base.BaseTestCase):
         self.assertEqual([self._FAKE_VHD_PATH], disk_files)
         self.assertEqual([self._FAKE_VOLUME_DRIVE_PATH], volume_drives)
 
+    @mock.patch.object(vmutils.VMUtils, '_get_vm_disks')
+    def test_get_vm_disks_by_instance_name(self, mock_get_vm_disks):
+        self._lookup_vm()
+        mock_get_vm_disks.return_value = mock.sentinel.vm_disks
+
+        vm_disks = self._vmutils.get_vm_disks(self._FAKE_VM_NAME)
+
+        self._vmutils._lookup_vm_check.assert_called_once_with(
+            self._FAKE_VM_NAME)
+        self.assertEqual(mock.sentinel.vm_disks, vm_disks)
+
     def test_get_vm_disks(self):
         mock_vm = self._lookup_vm()
         mock_vmsettings = [mock.MagicMock()]
@@ -222,11 +253,48 @@ class VMUtilsTestCase(base.BaseTestCase):
         self.assertRaises(exceptions.HyperVAuthorizationException,
                           self._vmutils.check_admin_permissions)
 
+    @mock.patch.object(vmutils.VMUtils, '_set_vm_vcpus')
+    @mock.patch.object(vmutils.VMUtils, '_set_vm_memory')
+    @mock.patch.object(vmutils.VMUtils, '_get_vm_setting_data')
+    def test_update_vm(self, mock_get_vm_setting_data, mock_set_mem,
+                       mock_set_vcpus):
+        self._lookup_vm()
+        mock_vm_settings = mock_get_vm_setting_data.return_value
+
+        self._vmutils.update_vm(
+            mock.sentinel.vm_name, mock.sentinel.memory_mb,
+            mock.sentinel.memory_per_numa, mock.sentinel.vcpus_num,
+            mock.sentinel.vcpus_per_numa, mock.sentinel.limit_cpu_features,
+            mock.sentinel.dynamic_mem_ratio)
+
+        mock_set_mem.assert_called_once_with(
+            mock_vm_settings, mock.sentinel.memory_mb,
+            mock.sentinel.memory_per_numa, mock.sentinel.dynamic_mem_ratio)
+        mock_set_vcpus.assert_called_once_with(
+            mock_vm_settings, mock.sentinel.vcpus_num,
+            mock.sentinel.vcpus_per_numa, mock.sentinel.limit_cpu_features)
+
+    @mock.patch.object(vmutils.VMUtils, '_set_vm_memory')
+    @mock.patch.object(vmutils.VMUtils, '_create_vm_obj')
+    def test_vnuma_create_vm(self, mock_create_vm_obj, mock_set_mem):
+        self._vmutils._vs_man_svc.DefineSystem.return_value = (
+            None, self._FAKE_JOB_PATH, self._FAKE_RET_VAL)
+        self._vmutils.create_vm(self._FAKE_VM_NAME,
+                                mock.sentinel.vnuma_enabled,
+                                self._VM_GEN,
+                                mock.sentinel.instance_path)
+
+        mock_create_vm_obj.assert_called_once_with(
+            self._FAKE_VM_NAME, mock.sentinel.vnuma_enabled,
+            self._VM_GEN, None, mock.sentinel.instance_path)
+        self.assertFalse(mock_set_mem.called)
+
     @mock.patch.object(vmutils.VMUtils, '_set_vm_vcpus')
     @mock.patch.object(vmutils.VMUtils, '_set_vm_memory')
     @mock.patch.object(vmutils.VMUtils, '_get_wmi_obj')
-    def test_create_vm(self, mock_get_wmi_obj, mock_set_mem, mock_set_vcpus):
-        mock_svc = self._vmutils._conn.Msvm_VirtualSystemManagementService()[0]
+    def test_old_create_vm(self, mock_get_wmi_obj, mock_set_mem,
+                           mock_set_vcpus):
+        mock_svc = self._vmutils._vs_man_svc
         getattr(mock_svc, self._DEFINE_SYSTEM).return_value = (
             None, self._FAKE_JOB_PATH, self._FAKE_RET_VAL)
 
@@ -246,10 +314,11 @@ class VMUtilsTestCase(base.BaseTestCase):
                                 mock.sentinel.instance_path)
 
         self.assertTrue(getattr(mock_svc, self._DEFINE_SYSTEM).called)
-        mock_set_mem.assert_called_with(mock_s, self._FAKE_MEMORY_MB,
+        mock_set_mem.assert_called_with(mock_s, self._FAKE_MEMORY_MB, None,
                                         self._FAKE_DYNAMIC_MEMORY_RATIO)
 
-        mock_set_vcpus.assert_called_with(mock_s, self._FAKE_VCPUS_NUM, False)
+        mock_set_vcpus.assert_called_with(mock_s, self._FAKE_VCPUS_NUM, None,
+                                          False)
 
     def test_get_vm_scsi_controller(self):
         self._prepare_get_vm_controller(self._vmutils._SCSI_CTRL_RES_SUB_TYPE)
@@ -358,16 +427,34 @@ class VMUtilsTestCase(base.BaseTestCase):
             mock_get_new_rsd.return_value, mock_vm)
 
     @mock.patch.object(vmutils.VMUtils, '_get_new_resource_setting_data')
-    def test_attach_volume_to_controller(self, mock_get_new_rsd):
+    @mock.patch.object(vmutils, 'wmi', create=True)
+    def _test_attach_volume_to_controller(self, mock_wmi, mock_get_new_rsd,
+                                          disk_serial=None):
         mock_vm = self._lookup_vm()
+        mock_diskdrive = mock.MagicMock()
+        jobutils = self._vmutils._jobutils
+        jobutils.add_virt_resource.return_value = [mock_diskdrive]
+        mock_wmi.WMI.return_value = mock_diskdrive
 
         self._vmutils.attach_volume_to_controller(
             self._FAKE_VM_NAME, self._FAKE_CTRL_PATH, self._FAKE_CTRL_ADDR,
-            self._FAKE_MOUNTED_DISK_PATH)
+            self._FAKE_MOUNTED_DISK_PATH, serial=disk_serial)
 
         self._vmutils._jobutils.add_virt_resource.assert_called_once_with(
             mock_get_new_rsd.return_value, mock_vm)
 
+        if disk_serial:
+            jobutils.modify_virt_resource.assert_called_once_with(
+                mock_diskdrive)
+            self.assertEqual(disk_serial, mock_diskdrive.ElementName)
+
+    def test_attach_volume_to_controller_without_disk_serial(self):
+        self._test_attach_volume_to_controller()
+
+    def test_attach_volume_to_controller_with_disk_serial(self):
+        self._test_attach_volume_to_controller(
+            disk_serial=mock.sentinel.serial)
+
     @mock.patch.object(vmutils.VMUtils, '_get_new_setting_data')
     def test_create_nic(self, mock_get_new_virt_res):
         mock_vm = self._lookup_vm()
@@ -402,7 +489,7 @@ class VMUtilsTestCase(base.BaseTestCase):
     def test_destroy_vm(self):
         self._lookup_vm()
 
-        mock_svc = self._vmutils._conn.Msvm_VirtualSystemManagementService()[0]
+        mock_svc = self._vmutils._vs_man_svc
         getattr(mock_svc, self._DESTROY_SYSTEM).return_value = (
             self._FAKE_JOB_PATH, self._FAKE_RET_VAL)
 
@@ -411,6 +498,38 @@ class VMUtilsTestCase(base.BaseTestCase):
         getattr(mock_svc, self._DESTROY_SYSTEM).assert_called_with(
             self._FAKE_VM_PATH)
 
+    @mock.patch.object(vmutils.VMUtils, '_get_vm_disks')
+    def test_get_vm_physical_disk_mapping(self, mock_get_vm_disks):
+        self._lookup_vm()
+        mock_phys_disk = self._create_mock_disks()[1]
+
+        expected_serial = mock_phys_disk.ElementName
+        expected_mapping = {
+            expected_serial: {
+                'resource_path': mock_phys_disk.Path_.return_value,
+                'mounted_disk_path': mock_phys_disk.HostResource[0]
+            }
+        }
+
+        mock_get_vm_disks.return_value = ([], [mock_phys_disk])
+
+        result = self._vmutils.get_vm_physical_disk_mapping(self._FAKE_VM_NAME)
+        self.assertEqual(expected_mapping, result)
+
+    @mock.patch.object(vmutils, 'wmi', create=True)
+    def test_set_disk_host_res(self, mock_wmi):
+        mock_diskdrive = mock_wmi.WMI.return_value
+
+        self._vmutils.set_disk_host_res(self._FAKE_RES_PATH,
+                                        self._FAKE_MOUNTED_DISK_PATH)
+
+        self._vmutils._jobutils.modify_virt_resource.assert_called_once_with(
+            mock_diskdrive)
+
+        mock_wmi.WMI.assert_called_once_with(moniker=self._FAKE_RES_PATH)
+        self.assertEqual(mock_diskdrive.HostResource,
+                         [self._FAKE_MOUNTED_DISK_PATH])
+
     def test_set_disk_host_resource(self):
         self._lookup_vm()
         mock_rasds = self._create_mock_disks()
@@ -538,6 +657,21 @@ class VMUtilsTestCase(base.BaseTestCase):
 
         self.assertEqual(mock_disk_2, physical_disk)
 
+    def test_get_dev_number_from_dev_name(self):
+        fake_physical_device_name = r'\\.\PhysicalDrive1'
+        expected_device_number = '1'
+
+        get_dev_number = self._vmutils.get_device_number_from_device_name
+        resulted_dev_number = get_dev_number(fake_physical_device_name)
+        self.assertEqual(expected_device_number, resulted_dev_number)
+
+    def test_get_device_number_from_invalid_device_name(self):
+        fake_physical_device_name = ''
+
+        self.assertRaises(exceptions.HyperVException,
+                          self._vmutils.get_device_number_from_device_name,
+                          fake_physical_device_name)
+
     def test_get_controller_volume_paths(self):
         self._prepare_mock_disk()
         mock_disks = {self._FAKE_RES_PATH: self._FAKE_HOST_RESOURCE}
@@ -599,6 +733,60 @@ class VMUtilsTestCase(base.BaseTestCase):
         else:
             self.assertEqual(old_serial_connection, ret_val)
 
+    def test_get_vm_serial_ports(self):
+        mock_vm = self._lookup_vm()
+        mock_vmsettings = [mock.MagicMock()]
+        mock_vm.associators.return_value = mock_vmsettings
+
+        fake_serial_port = mock.MagicMock()
+        fake_serial_port.ResourceSubType = (
+            self._vmutils._SERIAL_PORT_RES_SUB_TYPE)
+
+        mock_rasds = [fake_serial_port]
+        mock_vmsettings[0].associators.return_value = mock_rasds
+
+        ret_val = self._vmutils._get_vm_serial_ports(mock_vm)
+
+        mock_vmsettings[0].associators.assert_called_once_with(
+            wmi_result_class=self._vmutils._SERIAL_PORT_SETTING_DATA_CLASS)
+        self.assertEqual(mock_rasds, ret_val)
+
+    def test_set_vm_serial_port_conn(self):
+        mock_vm = self._lookup_vm()
+        mock_com_1 = mock.Mock()
+        mock_com_2 = mock.Mock()
+
+        self._vmutils._get_vm_serial_ports = mock.Mock(
+            return_value=[mock_com_1, mock_com_2])
+        self._vmutils._modify_virt_resource = mock.Mock()
+
+        self._vmutils.set_vm_serial_port_connection(
+            mock.sentinel.vm_name,
+            port_number=1,
+            pipe_path=mock.sentinel.pipe_path)
+
+        self.assertEqual([mock.sentinel.pipe_path], mock_com_1.Connection)
+        self._vmutils._modify_virt_resource.assert_called_once_with(
+            mock_com_1, mock_vm.path_())
+
+    def test_get_serial_port_conns(self):
+        self._lookup_vm()
+
+        mock_com_1 = mock.Mock()
+        mock_com_1.Connection = []
+
+        mock_com_2 = mock.Mock()
+        mock_com_2.Connection = [mock.sentinel.pipe_path]
+
+        self._vmutils._get_vm_serial_ports = mock.Mock(
+            return_value=[mock_com_1, mock_com_2])
+
+        ret_val = self._vmutils.get_vm_serial_port_connections(
+            mock.sentinel.vm_name)
+        expected_ret_val = [mock.sentinel.pipe_path]
+
+        self.assertEqual(expected_ret_val, ret_val)
+
     def test_set_vm_serial_port_connection(self):
         self._test_get_vm_serial_port_connection('new_serial_connection')
 
@@ -621,7 +809,7 @@ class VMUtilsTestCase(base.BaseTestCase):
             VirtualSystemType=self._vmutils._VIRTUAL_SYSTEM_TYPE_REALIZED)
 
     def test_modify_virtual_system(self):
-        mock_vs_man_svc = mock.MagicMock()
+        mock_vs_man_svc = self._vmutils._vs_man_svc
         mock_vmsetting = mock.MagicMock()
         fake_job_path = 'fake job path'
         fake_ret_val = 'fake return value'
@@ -629,8 +817,7 @@ class VMUtilsTestCase(base.BaseTestCase):
         mock_vs_man_svc.ModifySystemSettings.return_value = (fake_job_path,
                                                             fake_ret_val)
 
-        self._vmutils._modify_virtual_system(vs_man_svc=mock_vs_man_svc,
-                                             vmsetting=mock_vmsetting)
+        self._vmutils._modify_virtual_system(vmsetting=mock_vmsetting)
 
         mock_vs_man_svc.ModifySystemSettings.assert_called_once_with(
             SystemSettings=mock_vmsetting.GetText_(1))
@@ -639,8 +826,8 @@ class VMUtilsTestCase(base.BaseTestCase):
 
     @mock.patch.object(vmutils.VMUtils, '_get_wmi_obj')
     def _test_create_vm_obj(self, mock_get_wmi_obj, vm_path,
-                            dynamic_memory_ratio=1.0):
-        mock_vs_man_svc = mock.MagicMock()
+                            vnuma_enabled=True):
+        mock_vs_man_svc = self._vmutils._vs_man_svc
         mock_vs_data = mock.MagicMock()
         mock_job = mock.MagicMock()
         fake_job_path = 'fake job path'
@@ -656,11 +843,10 @@ class VMUtilsTestCase(base.BaseTestCase):
         mock_job.associators.return_value = ['fake vm path']
 
         response = self._vmutils._create_vm_obj(
-            vs_man_svc=mock_vs_man_svc,
             vm_name=fake_vm_name,
             vm_gen=constants.VM_GEN_2,
             notes='fake notes',
-            dynamic_memory_ratio=dynamic_memory_ratio,
+            vnuma_enabled=vnuma_enabled,
             instance_path=mock.sentinel.instance_path)
 
         if not vm_path:
@@ -679,11 +865,11 @@ class VMUtilsTestCase(base.BaseTestCase):
                          mock_vs_data.VirtualSystemSubType)
         self.assertFalse(mock_vs_data.SecureBootEnabled)
 
-        if dynamic_memory_ratio > 1:
-            self.assertFalse(mock_vs_data.VirtualNumaEnabled)
-
         mock_get_wmi_obj.assert_called_with('fake vm path')
 
+        self.assertEqual(vnuma_enabled, mock_vs_data.VirtualNumaEnabled)
+        self.assertEqual(self._vmutils._VIRTUAL_SYSTEM_SUBTYPE_GEN2,
+                         mock_vs_data.VirtualSystemSubType)
         self.assertEqual(mock_vs_data.Notes, 'fake notes')
         self.assertEqual(mock.sentinel.instance_path,
                          mock_vs_data.ConfigurationDataRoot)
@@ -702,8 +888,8 @@ class VMUtilsTestCase(base.BaseTestCase):
     def test_create_vm_obj_no_vm_path(self):
         self._test_create_vm_obj(vm_path=None)
 
-    def test_create_vm_obj_dynamic_memory(self):
-        self._test_create_vm_obj(vm_path=None, dynamic_memory_ratio=1.1)
+    def test_create_vm_obj_vnuma_disabled(self):
+        self._test_create_vm_obj(vm_path=None, vnuma_enabled=False)
 
     def test_list_instances(self):
         vs = mock.MagicMock()
@@ -843,7 +1029,6 @@ class VMUtilsTestCase(base.BaseTestCase):
                                 mock_modify_virtual_system):
         vm = mock_lookup_vm_check.return_value
         vs_data = mock_get_vm_setting_data.return_value
-        vs_svc = self._vmutils._conn.Msvm_VirtualSystemManagementService()[0]
 
         with mock.patch.object(self._vmutils,
                                '_set_secure_boot') as mock_set_secure_boot:
@@ -854,8 +1039,7 @@ class VMUtilsTestCase(base.BaseTestCase):
             mock_get_vm_setting_data.assert_called_once_with(vm)
             mock_set_secure_boot.assert_called_once_with(
                 vs_data, mock.sentinel.msft_ca_required)
-            mock_modify_virtual_system.assert_called_once_with(
-                vs_svc, vs_data)
+            mock_modify_virtual_system.assert_called_once_with(vs_data)
 
     @mock.patch.object(vmutils.VMUtils,
                        '_get_mounted_disk_resource_from_path')
@@ -950,15 +1134,13 @@ class VMUtilsTestCase(base.BaseTestCase):
                             mock_get_vm_setting_data):
         mock_vm = self._lookup_vm()
 
-        mock_svc = self._vmutils._conn.Msvm_VirtualSystemManagementService()[0]
         mock_vssd = mock_get_vm_setting_data.return_value
         fake_dev_boot_order = [mock.sentinel.BOOT_DEV1,
                                mock.sentinel.BOOT_DEV2]
         self._vmutils._set_boot_order_gen1(
             mock_vm.name, fake_dev_boot_order)
 
-        mock_modify_virt_syst.assert_called_once_with(
-            mock_svc, mock_vssd)
+        mock_modify_virt_syst.assert_called_once_with(mock_vssd)
         self.assertEqual(mock_vssd.BootOrder, tuple(fake_dev_boot_order))
 
     @mock.patch.object(vmutils.VMUtils, '_get_vm_setting_data')
@@ -975,7 +1157,6 @@ class VMUtilsTestCase(base.BaseTestCase):
         fake_dev_order = [fake_boot_dev1, fake_boot_dev2]
         mock_drive_to_boot_source.side_effect = fake_dev_order
         mock_vm = self._lookup_vm()
-        mock_svc = self._vmutils._conn.Msvm_VirtualSystemManagementService()[0]
         mock_vssd = mock_get_vm_setting_data.return_value
         old_boot_order = tuple([mock.sentinel.BOOT_SOURCE2,
                                 mock.sentinel.BOOT_SOURCE1,
@@ -987,6 +1168,5 @@ class VMUtilsTestCase(base.BaseTestCase):
 
         self._vmutils._set_boot_order_gen2(mock_vm.name, fake_dev_order)
 
-        mock_modify_virtual_system.assert_called_once_with(
-            mock_svc, mock_vssd)
+        mock_modify_virtual_system.assert_called_once_with(mock_vssd)
         self.assertEqual(expected_boot_order, mock_vssd.BootSourceOrder)
