@@ -14,15 +14,14 @@
 #    under the License.
 
 import mock
-from oslotest import base
 
 from os_win import exceptions
+from os_win.tests import test_base
 from os_win.utils.storage import smbutils
 
 
-class SMBUtilsTestCase(base.BaseTestCase):
-    @mock.patch.object(smbutils, 'wmi', create=True)
-    def setUp(self, mock_wmi):
+class SMBUtilsTestCase(test_base.OsWinBaseTestCase):
+    def setUp(self):
         super(SMBUtilsTestCase, self).setUp()
 
         self._smbutils = smbutils.SMBUtils()
